library ieee;
use ieee.std_logic_1164.all;

use work.verification_package.all;

entity advanced_testbench is
end entity advanced_testbench;

architecture bhv of advanced_testbench is

  -- 50 MHz clock frequency
  constant clk_period:        time                          := 20 ns;
        
  signal clk:                 std_logic                     := '0';
  signal rst_n:               std_logic                     := '1';
  signal rx:                  std_logic                     := '1';
  signal tx:                  std_logic;      
  signal transmitter_ready:   std_logic;      
  signal parity_error_flag:   std_logic;      
  signal btn_n:               std_logic                     := '1';
  signal switches:            std_logic_vector(7 downto 0)  := X"EA";
  signal data_received:       std_logic_vector(7 downto 0);
  signal parity_error:        boolean;
  signal ssd_digit_0:         std_logic_vector(7 downto 0);
  signal ssd_digit_1:         std_logic_vector(7 downto 0);

  -- ---------------------------------------------------
  -- Added for self checking capability
  -- ---------------------------------------------------
  -- check triggers
  signal check_1:             boolean                       := false;
  signal check_2:             boolean                       := false;
  signal check_3:             boolean                       := false;

begin

  L_CLOCK: process
  begin
    wait for clk_period / 2;
    clk <= not clk;
  end process;
  
  L_DUV:  entity work.estec_lab_top(rtl)
    port map (
      clk               => clk,
      rst_n             => rst_n,
      rx                => rx,
      tx                => tx,
      transmitter_ready => transmitter_ready,
      parity_error_flag => parity_error_flag,
      btn_n             => btn_n,
      switches          => switches,
      ssd_digit_0       => ssd_digit_0,
      ssd_digit_1       => ssd_digit_1
    );
  
  L_TEST_SEQUENCER: process
  begin

    -- active and deactive the reset signal
    wait for 69 ns; 
    rst_n <= '0';

    -- ---------------------------------------------------
    -- Added for self checking capability 
    -- ---------------------------------------------------
    -- Trigger check #1
    wait for 1 ns;
    check_1 <= true;
    
    wait for 65 ns; 
    rst_n <= '1';

    wait for 1 ms;
    
    -- ---------------------------------------------------
    -- Added for self checking capability 
    -- ---------------------------------------------------
    -- Trigger check #2

    -- pressing the button with bouncing
    bouncing_button(120 us, 7, btn_n);
    check_2 <= true;
    
    wait for 1 ms;

    -- ---------------------------------------------------
    -- Added for self checking capability 
    -- ---------------------------------------------------
    -- Trigger check #3
    check_3 <= true;

    -- send 0x2D
    gen_uart_frame(X"2D", 8.67 us, 1, False, 0 us, rx);
    
    wait for 1 ms;
    
    -- ---------------------------------------------------
    -- Add the following test sequence manually...
    -- ---------------------------------------------------

    -- send 0xE1
    gen_uart_frame(X"E1", 8.67 us, 1, False, 0 us, rx);

    wait;
  end process;

  L_CHECKS: process
  begin

    -- check #1
    wait until check_1 = true;
    if ssd_digit_0 /= "11111111" then
      report "--------------------------------------------------------------> Check #1: 7 segment display digit 0 ERROR"; 
    else
      report "--------------------------------------------------------------> Check #1: 7 segment display digit 0 OK";
    end if;
    
    if ssd_digit_1 /= "11111111" then
      report "--------------------------------------------------------------> Check #1: 7 segment display digit 1 ERROR";
    else
      report "--------------------------------------------------------------> Check #1: 7 segment display digit 1 OK";
    end if;
    
    if transmitter_ready /= '1' then
      report "--------------------------------------------------------------> Check #1: Transmitter ready flag ERROR";
    else
      report "--------------------------------------------------------------> Check #1: Transmitter ready flag OK";
    end if;
    
    if parity_error_flag /= '0' then
      report "--------------------------------------------------------------> Check #1: Parity error flag ERROR";
    else
      report "--------------------------------------------------------------> Check #1: Parity error flag OK";
    end if;

    -- check #2
    wait until check_2 = true;
    rec_uart_frame(8.67 us, 1, 600 us, tx, parity_error, data_received);
    
    if parity_error then
      report "--------------------------------------------------------------> Check #2: Parity ERROR"; 
    elsif data_received /= switches then
      report "--------------------------------------------------------------> Check #2: Data received ERROR";
    else
      report "--------------------------------------------------------------> Check #2: Data received OK";
    end if;

    -- check #3
    --wait until check_3 = true;
    --wait on ssd_digit_0'transaction;
    --if seven_segment_decoder(ssd_digit_0) /= X"1" then
    --  report "--------------------------------------------------------------> Check #3: 7 segment display digit 0 ERROR"; 
    --else
    --  report "--------------------------------------------------------------> Check #3: 7 segment display digit 0 OK"; 
    --end if;
    --if seven_segment_decoder(ssd_digit_1) /= X"E" then
    --  report "--------------------------------------------------------------> Check #3: 7 segment display digit 1 ERROR"; 
    --else
    --  report "--------------------------------------------------------------> Check #3: 7 segment display digit 1 OK"; 
    --end if;


  wait;
  end process;

end architecture bhv;