library ieee;
use ieee.std_logic_1164.all;

entity simple_testbench is
end entity simple_testbench;

architecture bhv of simple_testbench is

  -- 50 MHz clock frequency
  constant clk_period:        time                          := 20 ns;
        
  signal clk:                 std_logic                     := '0';
  signal rst_n:               std_logic                     := '1';
  signal rx:                  std_logic                     := '1';
  signal tx:                  std_logic;      
  signal transmitter_ready:   std_logic;      
  signal parity_error_flag:   std_logic;      
  signal btn_n:               std_logic                     := '1';
  signal switches:            std_logic_vector (7 downto 0) := X"EA";
  signal ssd_digit_0:         std_logic_vector (7 downto 0);
  signal ssd_digit_1:         std_logic_vector (7 downto 0);

  -- ---------------------------------------------------
  -- Added for self checking capability
  -- ---------------------------------------------------
  -- check triggers
  signal check_1:             boolean                       := false;
  signal check_2:             boolean                       := false;
  signal check_3:             boolean                       := false;

begin

  L_CLOCK: process
  begin
    wait for clk_period / 2;
    clk <= not clk;
  end process;
  
  L_DUV:  entity work.estec_lab_top(rtl)
    port map (
      clk               => clk,
      rst_n             => rst_n,
      rx                => rx,
      tx                => tx,
      transmitter_ready => transmitter_ready,
      parity_error_flag => parity_error_flag,
      btn_n             => btn_n,
      switches          => switches,
      ssd_digit_0       => ssd_digit_0,
      ssd_digit_1       => ssd_digit_1
    );
  
  L_TEST_SEQUENCER: process
  begin

    -- active and deactive the reset signal
    wait for 69 ns; 
    rst_n <= '0';

    -- ---------------------------------------------------
    -- Added for self checking capability 
    -- ---------------------------------------------------
    -- Trigger check #1
    wait for 1 ns;
    check_1 <= true;
    
    wait for 65 ns; 
    rst_n <= '1';

    wait for 1 ms;

    -- ---------------------------------------------------
    -- Added for self checking capability 
    -- ---------------------------------------------------
    
    -- pressing the button with bouncing
    for i in 0 to 6 loop
      wait for 120 us;
      btn_n <= not btn_n;
    end loop;

    -- Trigger check #2
    check_2 <= true;
    
    wait for 1 ms;

    -- ---------------------------------------------------
    -- Added for self checking capability 
    -- ---------------------------------------------------
    check_3 <= true;

    -- send 0x2D
    rx <= '0'; wait for 8.67 us;  -- start bit
    rx <= '1'; wait for 8.67 us;  -- bit 0
    rx <= '0'; wait for 8.67 us;  -- bit 1
    rx <= '1'; wait for 8.67 us;  -- bit 2
    rx <= '1'; wait for 8.67 us;  -- bit 3
    rx <= '0'; wait for 8.67 us;  -- bit 4
    rx <= '1'; wait for 8.67 us;  -- bit 5
    rx <= '0'; wait for 8.67 us;  -- bit 6
    rx <= '0'; wait for 8.67 us;  -- bit 7
    rx <= '1'; wait for 8.67 us;  -- parity bit
    rx <= '1'; wait for 8.67 us;  -- stop bit
    
    wait for 1 ms;
    
    -- ---------------------------------------------------
    -- Add the following test sequence manually...
    -- ---------------------------------------------------

    -- send 0xE1
    rx <= '0'; wait for 8.67 us;  -- start bit
    rx <= '1'; wait for 8.67 us;  -- bit 0
    rx <= '0'; wait for 8.67 us;  -- bit 1
    rx <= '0'; wait for 8.67 us;  -- bit 2
    rx <= '0'; wait for 8.67 us;  -- bit 3
    rx <= '0'; wait for 8.67 us;  -- bit 4
    rx <= '1'; wait for 8.67 us;  -- bit 5
    rx <= '1'; wait for 8.67 us;  -- bit 6
    rx <= '1'; wait for 8.67 us;  -- bit 7
    rx <= '1'; wait for 8.67 us;  -- parity bit
    rx <= '1'; wait for 8.67 us;  -- stop bit

    wait;
  end process;

  L_CHECKS: process
  --declaring variable to sample the ssd digit 0 and 1.
    variable ssd_digit_0_sample: std_logic_vector(7 downto 0);
    variable ssd_digit_1_sample: std_logic_vector(7 downto 0);
  begin

    -- check #1
    wait until check_1 = true;
    if ssd_digit_0 /= "11111111" then
      report "--------------------------------------------------------------> Check #1: 7 segment display digit 0 ERROR"; 
    else
      report "--------------------------------------------------------------> Check #1: 7 segment display digit 0 OK";
    end if;
    
    if ssd_digit_1 /= "11111111" then
      report "--------------------------------------------------------------> Check #1: 7 segment display digit 1 ERROR";
    else
      report "--------------------------------------------------------------> Check #1: 7 segment display digit 1 OK";
    end if;
    
    if transmitter_ready /= '1' then
      report "--------------------------------------------------------------> Check #1: Transmitter ready flag ERROR";
    else
      report "--------------------------------------------------------------> Check #1: Transmitter ready flag OK";
    end if;
    
    if parity_error_flag /= '0' then
      report "--------------------------------------------------------------> Check #1: Parity error flag ERROR";
    else
      report "--------------------------------------------------------------> Check #1: Parity error flag OK";
    end if;

    -- check #2
    wait until check_2 = true;
    wait until tx = '0'; -- start bit
    
    wait for 13 us;
    if tx /= '0' then report "--------------------------------------------------------------> Check # 2: TX LSB ERROR";
    else report "--------------------------------------------------------------> Check # 2: TX LSB OK"; end if; -- LSB
    
    wait for 8.67 us;
    if tx /= '1' then report "--------------------------------------------------------------> Check # 2: TX 2nd bit ERROR";
    else report "--------------------------------------------------------------> Check # 2: TX 2nd bit OK"; end if;

    wait for 8.67 us;
    if tx /= '0' then report "--------------------------------------------------------------> Check # 2: TX 3rd bit ERROR";
    else report "--------------------------------------------------------------> Check # 2: TX 3rd bit OK"; end if;

    wait for 8.67 us;
    if tx /= '1' then report "--------------------------------------------------------------> Check # 2: TX 4th bit ERROR"; 
    else report "--------------------------------------------------------------> Check # 2: TX 4th bit OK"; end if;

    wait for 8.67 us;
    if tx /= '0' then report "--------------------------------------------------------------> Check # 2: TX 5th bit ERROR";
    else report "--------------------------------------------------------------> Check # 2: TX 5th bit OK"; end if;

    wait for 8.67 us;
    if tx /= '1' then report "--------------------------------------------------------------> Check # 2: TX 6th bit ERROR";
    else report "--------------------------------------------------------------> Check # 2: TX 6th bit OK"; end if;

    wait for 8.67 us;
    if tx /= '1' then report "--------------------------------------------------------------> Check # 2: TX 7th bit ERROR";
    else report "--------------------------------------------------------------> Check # 2: TX 7th bit OK"; end if;

    wait for 8.67 us;
    if tx /= '1' then report "--------------------------------------------------------------> Check # 2: TX MSB ERROR"; 
    else report "--------------------------------------------------------------> Check # 2: TX MSB OK"; end if; -- MSB

    -- check #3
    -- this signal is not available in the PPR model!!!
    --wait until check_3 = true;
    --ssd_digit_0_sample := ssd_digit_0;
    --ssd_digit_1_sample := ssd_digit_1;
    --wait on <<signal .simple_testbench.L_DUV.frame_out: std_logic_vector (7 downto 0)>>'transaction;
    --if <<signal .simple_testbench.L_DUV.frame_out: std_logic_vector (7 downto 0)>> /= X"2D" then
    --  report "--------------------------------------------------------------> Check #3: RX data ERROR";
    --else
    --  report "--------------------------------------------------------------> Check #3: RX data OK";
    --end if;
    --if ssd_digit_0_sample /= ssd_digit_0 then
    --  report "--------------------------------------------------------------> Check #3: SSD digit 0 ERROR";
    --else
    --  report "--------------------------------------------------------------> Check #3: SSD digit 0 OK";
    --end if;
    --if ssd_digit_1_sample /= ssd_digit_1 then
    --  report "--------------------------------------------------------------> Check #3: SSD digit 1 ERROR";
    --else
    --  report "--------------------------------------------------------------> Check #3: SSD digit 1 OK";
    --end if;

  wait;
  end process;

end architecture bhv;